<?php


namespace IPS\videobox\api;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Category API
 */
class _categories extends \IPS\Node\Api\NodeController
{
	/**
	 * Class
	 */
	protected $class = 'IPS\videobox\Category';

	/**
	 * GET /videobox/categories
	 * Get list of categories
	 *
	 * @return		\IPS\Api\PaginatedResponse<IPS\videobox\Category>
	 */
	public function GETindex()
	{
		/* Return */
		return $this->_list();
	}

	/**
	 * GET /videobox/categories/{id}
	 * Get specific category
	 *
	 * @param		int		$id			ID Number
	 *
	 * @return		\IPS\Api\PaginatedResponse<IPS\videobox\Category>
	 */
	public function GETitem( $id )
	{
		/* Return */
		return $this->_view( $id );
	}

	/**
	 * POST /videobox/categories
	 * Create a category
	 *
	 * @apiparam	int|null	parent					The ID number of the parent the category should be created in. NULL for root.
	 * @apiparam	int			moderation				Require videos to be approved?
	 * @apiparam	int			comments				Enable comments?
	 * @apiparam	int			comment_moderation		Require comments to be approved?
	 * @apiparam	int			rating					Allow rating
	 * @apiparam	int			sharing					Allow sharing
	 * @apiparam	int			containerMode			Enable container mode
	 * @apiparam	int			reviews					Allow reviews?
	 * @apiparam	int			review_moderation		Require reviews to be approved?
	 *
	 * @return		\IPS\videobox\Category
	 */
	public function POSTindex()
	{
		if ( !\IPS\Request::i()->title )
		{
			throw new \IPS\Api\Exception( 'NO_TITLE', '1D365/2', 400 );
		}

		return new \IPS\Api\Response( 201, $this->_create()->apiOutput( $this->member ) );
	}

	/**
	 * POST /videobox/categories/{id}
	 * Edit a category
	 * 
	 * @apiparam	int|null	parent					The ID number of the parent the category should be created in. NULL for root.
	 * @apiparam	int			moderation				Require videos to be approved?
	 * @apiparam	int			comments				Enable comments?
	 * @apiparam	int			comment_moderation		Require comments to be approved?
	 * @apiparam	int			rating					Allow rating
	 * @apiparam	int			sharing					Allow sharing
	 * @apiparam	int			containerMode			Enable container mode
	 * @apiparam	int			reviews					Allow reviews?
	 * @apiparam	int			review_moderation		Require reviews to be approved?
	 *
	 * @return		\IPS\videobox\Category
	 */
	public function POSTitem( $id )
	{
		$class = $this->class;
		$category = $class::load( $id );
		
		return new \IPS\Api\Response( 200, $this->_createOrUpdate( $category )->apiOutput( $this->member ) );
	}

	/**
	 * DELETE /videobox/categories/{id}
	 * Delete a category
	 *
	 * @param		int		$id			ID Number
	 * @return		void
	 */
	public function DELETEitem( $id )
	{
		return $this->_delete( $id, \IPS\Request::i()->deleteChildrenOrMove ?? NULL );
	}

	/**
	 * Create or update node
	 *
	 * @param	\IPS\node\Model	$category				The node
	 * @return	\IPS\node\Model
	 */
	protected function _createOrUpdate( \IPS\node\Model $category )
	{
		if ( !\IPS\Request::i()->title )
		{
			throw new \IPS\Api\Exception( 'NO_TITLE', '', 400 );
		}

		foreach ( array( 'vcat_title' => "videobox_category_{$category->id}" ) as $fieldKey => $langKey )
		{
			if ( isset( \IPS\Request::i()->$fieldKey ) )
			{
				\IPS\Lang::saveCustom( 'videobox', $langKey, \IPS\Request::i()->$fieldKey );

				if ( $fieldKey === 'title' )
				{
					$category->name_furl = \IPS\Http\Url\Friendly::seoTitle( \IPS\Request::i()->$fieldKey );
				}
			}
		}
		

		$category->parent = (int) \IPS\Request::i()->parent?: 0;

		foreach ( array( 'moderation', 'comments', 'comment_moderation', 'rating', 'sharing', 'containerMode', 'noformat', 'hideinfo', 'reviews', 'review_moderation', 'showAlphabet', 'offDescription' ) as $k )
		{
			if ( isset( \IPS\Request::i()->$k ) )
			{
				$category->bitoptions[ $k ] = \IPS\Request::i()->$k;
			}
		}

		return parent::_createOrUpdate( $category );
	}
}