<?php

namespace IPS\videobox\api;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Videos API
 */
class _videos extends \IPS\Content\Api\ItemController
{
	/**
	 * Class
	 */
	protected $class = 'IPS\videobox\Video';
	
	/**
	 * GET /videobox/videos
	 * Get list of videos
	 *
	 * @note		For requests using an OAuth Access Token for a particular member, only videos the authorized user can view will be included
	 * @apiparam	string	categories		Comma-delimited list of category IDs
	 * @apiparam	string	authors			Comma-delimited list of member IDs - if provided, only videos started by those members are returned
	 * @apiparam	int		locked			If 1, only videos which are locked are returned, if 0 only unlocked
	 * @apiparam	int		hidden			If 1, only videos which are hidden are returned, if 0 only not hidden
	 * @apiparam	int		pinned			If 1, only videos which are pinned are returned, if 0 only not pinned
	 * @apiparam	int		featured		If 1, only videos which are featured are returned, if 0 only not featured
	 * @apiparam	string	sortBy			What to sort by. Can be 'date' for creation date, 'title' or leave unspecified for ID
	 * @apiparam	string	sortDir			Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page			Page number
	 * @apiparam	int		perPage			Number of results per page - defaults to 25
	 * @return		\IPS\Api\PaginatedResponse<IPS\videobox\Video>
	 */
	public function GETindex()
	{
		/* Where clause */
		$where = array();
				
		/* Return */
		return $this->_list( $where, 'categories' );
	}
	
	/**
	 * GET /videobox/videos/{id}
	 * View information about a specific video
	 *
	 * @param		int		$id				ID Number
	 * @throws		2BIMVB119/1	INVALID_ID		The video ID does not exist or the authorized user does not have permission to view it
	 * @return		\IPS\videobox\Video
	 */
	public function GETitem( $id )
	{
		try
		{
			$video = \IPS\videobox\Video::load( $id );
			if ( $this->member and !$video->can( 'read', $this->member ) )
			{
				throw new \OutOfRangeException;
			}
			return new \IPS\Api\Response( 200, $video->apiOutput( $this->member ) );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '2BIMVB119/1', 404 );
		}
	}
	

	/**
	 * GET /videobox/videos/{id}/comments
	 * Get comments on a video
	 *
	 * @param		int		$id			ID Number
	 * @apiparam	int		hidden		If 1, only comments which are hidden are returned, if 0 only not hidden
	 * @apiparam	string	sortDir		Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page		Page number
	 * @apiparam	int		perPage		Number of results per page - defaults to 25
	 * @throws		2BIMVB119/2	INVALID_ID	The video ID does not exist or the authorized user does not have permission to view it
	 * @return		\IPS\Api\PaginatedResponse<IPS\videobox\Video\Comment>
	 */
	public function GETitem_comments( $id )
	{
		try
		{
			return $this->_comments( $id, 'IPS\videobox\Video\Comment' );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '2BIMVB119/2', 404 );
		}
	}
	
	/**
	 * GET /videobox/videos/{id}/reviews
	 * Get reviews on a video
	 *
	 * @param		int		$id			ID Number
	 * @apiparam	int		hidden		If 1, only comments which are hidden are returned, if 0 only not hidden
	 * @apiparam	string	sortDir		Sort direction. Can be 'asc' or 'desc' - defaults to 'asc'
	 * @apiparam	int		page		Page number
	 * @apiparam	int		perPage		Number of results per page - defaults to 25
	 * @throws		2BIMVB119/3	INVALID_ID	The video ID does not exist or the authorized user does not have permission to view it
	 * @return		\IPS\Api\PaginatedResponse<IPS\videobox\Video\Review>
	 */
	public function GETitem_reviews( $id )
	{
		try
		{
			return $this->_comments( $id, 'IPS\videobox\Video\Review' );
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '2BIMVB119/3', 404 );
		}
	}
	
	/**
	 * DELETE /videobox/videos/{id}
	 * Delete a video
	 *
	 * @param		int		$id			ID Number
	 * @throws		2BIMVB119/5	INVALID_ID	The video ID does not exist
	 * @throws		2BIMVB119/P	NO_PERMISSION	The authorized user does not have permission to delete the video
	 * @return		void
	 */
	public function DELETEitem( $id )
	{
		try
		{
			$item = \IPS\videobox\Video::load( $id );
			if ( $this->member and !$item->canDelete( $this->member ) )
			{
				throw new \IPS\Api\Exception( 'NO_PERMISSION', '2G316/G', 404 );
			}
			
			$item->delete();
		}
		catch ( \OutOfRangeException $e )
		{
			throw new \IPS\Api\Exception( 'INVALID_ID', '2BIMVB119/P', 404 );
		}
	}
}