<?php

namespace IPS\videobox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Videobox Category Node
 */
class _Category extends \IPS\Node\Model implements \IPS\Node\Permissions
{
	use \IPS\Content\ClubContainer;
	
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Default Values
	 */
	protected static $defaultValues = NULL;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'videobox_categories';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'vcat_';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';
	
	/**
	 * @brief	[Node] Parent ID Database Column
	 */
	public static $databaseColumnParent = 'parent';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'videobox_cat_manage';
				
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = false;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 * @code
	 	array(
	 		'app'		=> 'core',				// The application key which holds the restrictrions
	 		'module'	=> 'foo',				// The module key which holds the restrictions
	 		'map'		=> array(				// [Optional] The key for each restriction - can alternatively use "prefix"
	 			'add'			=> 'foo_add',
	 			'edit'			=> 'foo_edit',
	 			'permissions'	=> 'foo_perms',
	 			'delete'		=> 'foo_delete'
	 		),
	 		'all'		=> 'foo_manage',		// [Optional] The key to use for any restriction not provided in the map (only needed if not providing all 4)
	 		'prefix'	=> 'foo_',				// [Optional] Rather than specifying each  key in the map, you can specify a prefix, and it will automatically look for restrictions with the key "[prefix]_add/edit/permissions/delete"
	 * @encode
	 */
	protected static $restrictions = array(
		'app'		=> 'videobox',
		'module'	=> 'video',
		'prefix' 	=> 'categories_',
		'all'		=> 'categories_manage',
	);
	
	/** 
	 * @brief	[Node] App for permission index
	 */
	public static $permApp = 'videobox';
	
	/** 
	 * @brief	[Node] Type for permission index
	 */
	public static $permType = 'category';
	
	/**
	 * @brief	The map of permission columns
	 */
	public static $permissionMap = array(
		'view' 		=> 'view',
		'read'		=> 2,
		'add'		=> 3,
		'reply'		=> 4,
		'review'	=> 5		
	);

	/** 
	 * @brief	[Node] Moderator Permission
	 */
	public static $modPerm = 'videobox_categories';
	
	/**
	 * Columns needed to query for search result / stream view
	 *
	 * @return	array
	 */
	public static function basicDataColumns()
	{
		$return = parent::basicDataColumns();
		$return[] = 'vcat_club_id';
		return $return;
	}
	
	
	/**
	 * @brief	Bitwise values for members_bitoptions field
	 */
	public static $bitOptions = array(
		'bitoptions' => array(
			'bitoptions' => array(
				'moderation'			=> 1,	// Require videos to be approved?
				'comments'				=> 2,	// Enable comments?
				'comment_moderation'	=> 4,	// Require comments to be approved?
				'rating'				=> 8, 	// Allow rating
				'sharing'				=> 16,  // Allow sharing
				# 32 is deprecated
				'containerMode'			=> 64,  // Enable container mode
				'noformat'				=> 128,  // Remove format in video			
				'hideinfo'				=> 256,  // Hide the info in video
				'reviews'				=> 512, // Enable reviews?
				'review_moderation'		=> 1024, // Require reviews to be approved?
				'offDescription'		=> 2048, // Disable discription
				'showAlphabet'			=> 4096, // Disable alphabet
			)
		)
	);	

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'videobox_category_';
	
	
	/**
	 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
	 */
	public static $permissionLangPrefix = 'videobox_perm_';
		
	/**
	 * @brief	Content Item Class
	 */
	public static $contentItemClass = 'IPS\videobox\Video';
			
	/**
	 * @brief	[Node] Enabled/Disabled Column
	 */
	public static $databaseColumnEnabledDisabled = 'open';
	
	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->open;
	}
	
	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		$this->open	= $enabled;
	}

	/**
	 * Get custom fields
	 *
	 * @return	array
	 */
	protected function get_cfields()
	{
		$fields = array();
		foreach ( explode( ',', $this->_data['cfields'] ) as $fieldId )
		{
			try
			{
				$fields[ $fieldId ] = \IPS\videobox\Field::load( $fieldId );
			}
			catch ( \OutOfRangeException $e ) { }
		}
		
		return $fields;
	}
	
	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{
		if ( $column === '_title' )
		{
			$column	= 'vcat_title';
		}

		if( $order == '_title' )
		{
			$order	= 'vcat_title';
		}

		return parent::search( $column, $query, $order, $where );
	}
	

	/**
	 * Get sort order
	 *
	 * @return	string
	 */
	public function get__sortBy()
	{
		return $this->sortorder;
	}

	/**
	 * Get sort order
	 *
	 * @return	string
	 */
	public function get__sortOrder()
	{
		return $this->sortorder == 'title' ? 'ASC' : parent::get__sortOrder();
	}
	
	/**
	 * Check permissions on any node
	 *
	 * For example - can be used to check if the user has
	 * permission to create content in any node to determine
	 * if there should be a "Submit" button
	 *
	 * @param	mixed								$permission		A key which has a value in static::$permissionMap['view'] matching a column ID in core_permission_index
	 * @param	\IPS\Member|\IPS\Member\Group|NULL	$member			The member or group to check (NULL for currently logged in member)
	 * @return	bool
	 * @throws	\OutOfBoundsException	If $permission does not exist in static::$permissionMap
	 */
	public static function canOnAny( $permission, $member=NULL, $where = array(), $considerPostBeforeRegistering = TRUE )
	{
		$member	= ( $member === NULL ) ? \IPS\Member::loggedIn() : $member;

		return parent::canOnAny( $permission, $member, $where );
	}
	
	/**
	 * Check permissions
	 *
	 * @param	mixed								$permission		A key which has a value in static::$permissionMap['view'] matching a column ID in core_permission_index
	 * @param	\IPS\Member|\IPS\Member\Group|NULL	$member			The member or group to check (NULL for currently logged in member)
	 * @return	bool
	 * @throws	\OutOfBoundsException	If $permission does not exist in static::$permissionMap
	 */
	public function can( $permission, $member=NULL, $considerPostBeforeRegistering = TRUE )
	{
		if ( \in_array( $permission, array( 'add', 'reply', 'review' ) ) )
		{
			if ( $this->bitoptions['containerMode'] == 1 )
			{
				return FALSE;
			}
		}
		
		return parent::can( $permission, $member );
	}	
	
	/**
	 * @brief	Cached URL
	 */
	protected $_url	= NULL;
	
	/**
	 * @brief	URL Base
	 */
	public static $urlBase = 'app=videobox&module=video&controller=browse&id=';
	
	/**
	 * @brief	URL Base
	 */
	public static $urlTemplate = 'videobox_cat';
	
	/**
	 * @brief	SEO Title Column
	 */
	public static $seoTitleColumn = 'title_furl';

	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->addTab( 'vcat_tab_settings' );
		$form->addHeader( 'vcat_tab_settings' );
		$form->add( new \IPS\Helpers\Form\Translatable( 'vcat_title', NULL, TRUE, array( 'app' => 'videobox', 'key' => ( $this->id ? "videobox_category_{$this->id}" : NULL ) ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_containerMode', $this->id ? $this->bitoptions['containerMode'] : FALSE, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\Node( 'vcat_parent', $this->parent ?: NULL, FALSE, array(
			'class'		      => '\IPS\videobox\Category',
			'disabled'	      => false,
			'zeroVal'         => 'node_no_parent',
			'permissionCheck' => function( $node )
			{
				if ( ! isset( \IPS\Request::i()->id ) )
				{
					return true;
				}
				
				return $node->id != \IPS\Request::i()->id;
			}
		) ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'vcat_sortorder', $this->sortorder ?: 'updated', FALSE, array( 'options' => array( 
			'updated' 		=> 'sort_updated', 
			'last_comment' 	=> 'last_reply', 
			'title' 		=> 'videobox_title', 
			'rating' 		=> 'sort_rating', 
			'date' 			=> 'sort_date', 
			'num_comments' 	=> 'sort_num_comments', 
			'num_reviews' 	=> 'sort_num_reviews', 
			'views' 		=> 'sort_num_views', 
			'online' 		=> 'sort_online' 
		) ), NULL, NULL, NULL, 'vcat_sortorder' ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_rating', $this->id ? $this->bitoptions['rating'] : TRUE, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_sharing', $this->id ? $this->bitoptions['sharing'] : TRUE, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Number( 'vcat_videoPerPage', $this->id ? $this->videoPerPage : 30, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_showAlphabet', $this->bitoptions['showAlphabet'] ? $this->bitoptions['showAlphabet'] : 0, FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_view_in_popup', $this->view_in_popup, FALSE, array( 'togglesOn' => array( 'vcat_view_in_popup_device' ) ) ) );
		//$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vcat_view_in_popup_device', isset( $this->view_in_popup_device ) ? explode(',', $this->view_in_popup_device) : FALSE, FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'vcat_view_in_popup_device' ) );
		
		$form->addHeader( 'vcat_header_moderation' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_moderation', $this->bitoptions['moderation'], FALSE, array() ) );	
				
		if ( \IPS\Settings::i()->tags_enabled )
		{
			$form->addHeader( 'vcat_category_tags' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_tags_disabled', !$this->tags_disabled, FALSE, array( 'togglesOn' => array( 'vcat_tags_noprefixes', 'vcat_tags_predefined', 'vcat_tags_filter_on' ) ), NULL, NULL, NULL, 'vcat_tags_disabled' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_tags_noprefixes', !$this->tags_noprefixes, FALSE, array(), NULL, NULL, NULL, 'vcat_tags_noprefixes' ) );
			
			if ( !\IPS\Settings::i()->tags_open_system )
			{
				$form->add( new \IPS\Helpers\Form\Text( 'vcat_tags_predefined', $this->tags_predefined, FALSE, array( 'autocomplete' => array( 'unique' => 'true' ), 'nullLang' => 'vcat_tags_predefined_unlimited' ), NULL, NULL, NULL, 'vcat_tags_predefined' ) );
			}
			
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_tags_filter_on', $this->tags_filter_on, FALSE, array(), NULL, NULL, NULL, 'vcat_tags_filter_on' ) );
		}

		$form->addHeader( 'vcat_header_comment' );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_comments', $this->id ? $this->bitoptions['comments'] : TRUE, FALSE, array( 'togglesOn' => array( 'vcat_bitoptions_comment_moderation' ) ), NULL, NULL, NULL, 'vcat_bitoptions_comments' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_comment_moderation', $this->bitoptions['comment_moderation'], FALSE, array(), NULL, NULL, NULL, 'vcat_bitoptions_comment_moderation' ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_reviews', $this->id ? $this->bitoptions['reviews'] : TRUE, FALSE, array( 'togglesOn' => array( 'vcat_bitoptions_review_moderation' ) ), NULL, NULL, NULL, 'vcat_bitoptions_reviews' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_review_moderation', $this->bitoptions['review_moderation'], FALSE, array(), NULL, NULL, NULL, 'vcat_bitoptions_review_moderation' ) );		
							
		$customFields = array();
		foreach ( \IPS\Db::i()->select( 'cf_id', 'videobox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "videobox_field_{$fieldId}" );
		}
		
		if ( !empty( $customFields ) )
		{
			$form->addTab('vcat_customfields_header');
			$form->addHeader('vcat_customfields_header');			
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_cfields', explode( ',', $this->_data['cfields'] ), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ) ) );
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_hidefields', explode( ',', $this->hidefields ), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ) ) );
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_cfields_pos', explode( ',', $this->cfields_pos ), FALSE, array( 'options' => array( 'main' => 'vcat_cfields_pos_main', 'sidebar' => 'vcat_cfields_pos_sidebar' ) ) ) );
			
			$form->addHeader('vcat_fieldsOnIMG_header');
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_fieldsOnTopIMG', explode( ',', $this->fieldsOnTopIMG ), FALSE, array( 'options' => $customFields, 'multiple' => FALSE ) ) );
			$form->add( new \IPS\Helpers\Form\TextArea( 'vcat_fieldsOnTopIMG_format', $this->fieldsOnTopIMG_format, FALSE, array() ) );
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_fieldsOnIMG', explode( ',', $this->fieldsOnIMG ), FALSE, array( 'options' => $customFields, 'multiple' => FALSE ) ) );
			$form->add( new \IPS\Helpers\Form\TextArea( 'vcat_fieldsOnIMG_format', $this->fieldsOnIMG_format, FALSE, array() ) );
			
			$form->addHeader('vcat_protectField');
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_protectField', explode( ',', $this->protectField ), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ) ) );
			$viewprotect = array( 	'logintoview' 			=> 'vb_cfields_logintoview',
									'liketoview' 			=> 'vb_cfields_liketoview',
									'loginliketoview'		=> 'vb_cfields_loginliketoview',
			);
			$form->add( new \IPS\Helpers\Form\Radio( 'vcat_viewprotectField', $this->viewprotectField ? $this->viewprotectField : "logintoview", FALSE, array( 'options' => $viewprotect ) ) );				
		}
		
		$form->addTab( 'vcat_tab_viewing' );
		$form->addHeader( 'vcat_tab_viewing' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_videoTab', $this->id ? $this->videoTab : 0, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_hideinfo', $this->bitoptions['hideinfo'], FALSE, array() ) );	
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_offDescription', $this->bitoptions['offDescription'] ? $this->bitoptions['offDescription'] : 0, FALSE, array( 'togglesOff' => array( 'vcat_bitoptions_noformat', 'vcat_contentformat' ) ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_noformat', $this->bitoptions['noformat'], FALSE, array(), NULL, NULL, NULL, 'vcat_bitoptions_noformat' ) );		
		$form->add( new \IPS\Helpers\Form\TextArea( 'vcat_contentformat', $this->id ? $this->contentformat : '', FALSE, array( 'rows' => 10, 'placeholder' => "<strong>{content}</strong>" ), NULL, NULL, NULL, 'vcat_contentformat' ) );			

		$form->addHeader('vcat_videopreview');
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_videopreview', $this->videopreview, FALSE, array() ) );
		if ( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_videopreview_fields', explode( ',', $this->videopreview_fields ), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ) ) );
		}	
		
		/* NEXUS */
		if ( \IPS\Application::appIsEnabled( 'nexus' ) )
		{
			$form->addTab( 'paid_video_settings' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_nexus_on', $this->nexus_on, FALSE, array( 'togglesOn' => array( 'vcat_nexus_tax', 'vcat_nexus_percent', 'vcat_nexus_transfee', 'vcat_nexus_gateways', 'vcat_nexus_can_add' ) ) ) );		
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_nexus_can_add', $this->nexus_can_add ? array_filter( explode( ',', $this->nexus_can_add ) ) : FALSE, FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ), NULL, NULL, NULL, 'vcat_nexus_can_add' ) );			
			$form->add( new \IPS\Helpers\Form\Node( 'vcat_nexus_tax', $this->nexus_tax ?:0, FALSE, array( 'class' => '\IPS\nexus\Tax', 'zeroVal' => 'do_not_tax' ), NULL, NULL, NULL, 'vcat_nexus_tax' ) );
			$form->add( new \IPS\Helpers\Form\Number( 'vcat_nexus_percent', $this->nexus_percent, FALSE, array( 'min' => 0, 'max' => 100 ), NULL, NULL, '%', 'vcat_nexus_percent' ) );
			$form->add( new \IPS\nexus\Form\Money( 'vcat_nexus_transfee', json_decode( $this->nexus_transfee, TRUE ), FALSE, array(), NULL, NULL, NULL, 'vcat_nexus_transfee' ) );
			$form->add( new \IPS\Helpers\Form\Node( 'vcat_nexus_gateways', ( $this->nexus_gateways ) ? explode( ',', $this->nexus_gateways ) : 0, FALSE, array( 'class' => '\IPS\nexus\Gateway', 'zeroVal' => 'no_restriction', 'multiple' => TRUE ), NULL, NULL, NULL, 'vcat_nexus_gateways' ) );
		}
		
		/* POINTS */
        $form->addTab( 'vb_conf_points_tab' );
        $form->add( new \IPS\Helpers\Form\YesNo( 'vcat_points_on', $this->id ? $this->points_on : 0, FALSE, array( 'togglesOn' => array( 'vcat_points_video', 'vcat_points_comment', 'vcat_points_review', 'vcat_points_reaction' ) ) ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vcat_points_video', $this->id ? $this->points_video : -1, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vcat_points_video' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vcat_points_comment', $this->id ? $this->points_comment : -1, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vcat_points_comment' ) );
        $form->add( new \IPS\Helpers\Form\Number( 'vcat_points_review', $this->id ? $this->points_review : -1, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vcat_points_review' ) );			
        $form->add( new \IPS\Helpers\Form\Number( 'vcat_points_reaction', $this->id ? $this->points_reaction : -1, FALSE, array( 'min' => 1, 'unlimited' => -1, 'unlimitedLang' => 'vb_points_disabled' ), NULL, NULL, NULL, 'vcat_points_reaction' ) );			
		
		/* FORUMS */
		if ( \IPS\Application::appIsEnabled( 'forums' ) )
		{
			if ( $this->id )
			{
				$rebuildUrl = \IPS\Http\Url::internal( 'app=videobox&module=video&controller=categories&id=' . $this->id . '&do=rebuildTopicContent' );
			}

			$form->addTab( 'vb_forums_integration' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_forum_on', $this->forum_id, FALSE, array( 'disableCopy' => TRUE, 'togglesOn' => array(
				'vcat_forum_id',
				'vcat_topic_prefix',
				'vcat_topic_suffix',
				'vcat_topic_delete',
			) ), NULL, NULL, $this->id ? \IPS\Member::loggedIn()->language()->addToStack( 'vb_category_topic_rebuild', NULL, array( 'sprintf' => array( $rebuildUrl ) ) ) : NULL ) );
			$form->add( new \IPS\Helpers\Form\Node( 'vcat_forum_id', $this->forum_id ? $this->forum_id  : NULL, FALSE, array( 'class' => 'IPS\forums\Forum', 'permissionCheck' => function ( $forum ) { return $forum->sub_can_post and !$forum->redirect_url; } ), NULL, NULL, NULL, 'vcat_forum_id' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'vcat_topic_prefix', $this->topic_prefix, FALSE, array( 'trim' => FALSE ), NULL, NULL, NULL, 'vcat_topic_prefix' ) );
			$form->add( new \IPS\Helpers\Form\Text( 'vcat_topic_suffix', $this->topic_suffix, FALSE, array( 'trim' => FALSE ), NULL, NULL, NULL, 'vcat_topic_suffix' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_topic_delete', $this->id ? $this->topic_delete : NULL, FALSE, array(), NULL, NULL, NULL, 'vcat_topic_delete' ) );
		}		
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function formatFormValues( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}
		
		/* Save lang for title */
		foreach ( array( 'vcat_title' => "videobox_category_{$this->id}" ) as $fieldKey => $langKey )
		{
			if ( isset( $values[ $fieldKey ] ) )
			{
				\IPS\Lang::saveCustom( 'videobox', $langKey, $values[ $fieldKey ] );
				
				if ( $fieldKey === 'vcat_title' )
				{
					$this->title = $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ];
					$this->title_furl = \IPS\Http\Url::seoTitle( $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ] );
				}
				
				unset( $values[ $fieldKey ] );
			}
		}		
		
		# Tags - Inverted for legacy reasons
		foreach ( array( 'vcat_tags_disabled', 'vcat_tags_noprefixes' ) as $k )
		{
			if ( isset( $values[ $k ] ) )
			{
				$values[ $k ] = !$values[ $k ];
			}
		}
		
		/* Save bitoptions */
		foreach ( array( 'moderation', 'comments', 'comment_moderation', 'rating', 'sharing', 'containerMode', 'noformat', 'hideinfo', 'reviews', 'review_moderation', 'showAlphabet', 'offDescription' ) as $k )
		{
			if ( isset( $values[ "vcat_bitoptions_{$k}" ] ) )
			{
				$this->bitoptions[ $k ] = $values["vcat_bitoptions_{$k}"];
				unset( $values["vcat_bitoptions_{$k}"] );
			}
		}
		
		/* Avoid "parent cannot be null" error if no parent selected. */
		if ( isset( $values['vcat_parent'] ) )
		{
			$values['vcat_parent'] = $values['vcat_parent'] ? \intval( $values['vcat_parent']->id ) : 0;
		}		
		
		/* Nexus */
		$values['vcat_nexus_transfee'] = json_encode( $values['vcat_nexus_transfee'] );
		
		/* Forums */
		if ( isset( $values['vcat_forum_on'] ) and !$values['vcat_forum_on'] )
		{
			$values['vcat_forum_id'] = 0;
		}
		
		if( isset( $values['vcat_forum_id'] ) AND $values['vcat_forum_id'] )
		{
			$values['vcat_forum_id'] = ( $values['vcat_forum_id'] instanceof \IPS\Node\Model ) ? \intval( $values['vcat_forum_id']->id ) : \intval( $values['vcat_forum_id'] );
		}

		if( array_key_exists( 'vcat_forum_on', $values ) )
		{
			unset( $values['vcat_forum_on'] );
		}
		
		/* return */
		return $values;
	}

	
	/* !Clubs */
	
	/**
	 * Get acp language string
	 *
	 * @return	string
	 */
	public static function clubAcpTitle()
	{
		return 'videobox_categories';
	}
	
	/**
	 * Set form for creating a node of this type in a club
	 *
	 * @param	\IPS\Helpers\Form	$form	Form object
	 * @return	void
	 */
	public function clubForm( \IPS\Helpers\Form $form )
	{
		$itemClass = static::$contentItemClass;
		$form->class = "ipsForm_horizontal";

		$form->addTab( 'vcat_tab_settings' );
		$form->addHeader( 'vcat_tab_settings' );
		$form->add( new \IPS\Helpers\Form\Text( 'club_node_name', $this->_id ? $this->_title : \IPS\Member::loggedIn()->language()->addToStack( $itemClass::$title . '_pl' ), TRUE, array( 'maxLength' => 255 ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_rating', $this->_id ? $this->bitoptions['rating'] : TRUE, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_sharing', $this->_id ? $this->bitoptions['sharing'] : TRUE, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Number( 'vcat_videoPerPage', $this->id ? $this->videoPerPage : 30, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_showAlphabet', $this->bitoptions['showAlphabet'] ? $this->bitoptions['showAlphabet'] : 0, FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_view_in_popup', $this->view_in_popup, FALSE, array( 'togglesOn' => array( 'vcat_view_in_popup_device' ) ) ) );
		//$form->add( new \IPS\Helpers\Form\CheckboxSet( 'vcat_view_in_popup_device', isset( $this->view_in_popup_device ) ? explode(',', $this->view_in_popup_device) : FALSE, FALSE, array( 'options' => array( 'Phone' => \IPS\Member::loggedIn()->language()->addToStack( 'device_phone' ), 'Tablet' => \IPS\Member::loggedIn()->language()->addToStack( 'device_tablet' ), 'Desktop' => \IPS\Member::loggedIn()->language()->addToStack( 'device_desktop' ) ) ), NULL, NULL, NULL, 'vcat_view_in_popup_device' ) );
	
		$form->addHeader( 'vcat_header_comment' );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_comments', $this->_id ? $this->bitoptions['comments'] : TRUE, FALSE, array( 'togglesOn' => array( 'vcat_bitoptions_comment_moderation' ) ), NULL, NULL, NULL, 'vcat_bitoptions_comments' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_reviews', $this->_id ? $this->bitoptions['reviews'] : TRUE, FALSE, array( 'togglesOn' => array( 'vcat_bitoptions_review_moderation' ) ), NULL, NULL, NULL, 'vcat_bitoptions_reviews' ) );
				
		if ( \IPS\Settings::i()->tags_enabled )
		{
			$form->addHeader( 'vcat_category_tags' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_tags_disabled', !$this->tags_disabled, FALSE, array( 'togglesOn' => array( 'vcat_tags_noprefixes', 'vcat_tags_predefined', 'vcat_tags_filter_on' ) ), NULL, NULL, NULL, 'vcat_tags_disabled' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_tags_noprefixes', !$this->tags_noprefixes, FALSE, array(), NULL, NULL, NULL, 'vcat_tags_noprefixes' ) );
			
			if ( !\IPS\Settings::i()->tags_open_system )
			{
				$form->add( new \IPS\Helpers\Form\Text( 'vcat_tags_predefined', $this->tags_predefined, FALSE, array( 'autocomplete' => array( 'unique' => 'true' ), 'nullLang' => 'vcat_tags_predefined_unlimited' ), NULL, NULL, NULL, 'vcat_tags_predefined' ) );
			}
			
			$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_tags_filter_on', $this->tags_filter_on, FALSE, array(), NULL, NULL, NULL, 'vcat_tags_filter_on' ) );
		}
		
		$customFields = array();
		foreach ( \IPS\Db::i()->select( 'cf_id', 'videobox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "videobox_field_{$fieldId}" );
		}
		
		if ( !empty( $customFields ) )
		{
			$form->addTab('vcat_customfields_header');
			$form->addHeader('vcat_customfields_header');			
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_cfields', explode( ',', $this->_data['cfields'] ), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ) ) );
			$form->add( new \IPS\Helpers\Form\Select( 'vcat_cfields_pos', explode( ',', $this->cfields_pos ), FALSE, array( 'options' => array( 'main' => 'vcat_cfields_pos_main', 'sidebar' => 'vcat_cfields_pos_sidebar' ) ) ) );			
		}
		
		$form->addTab( 'vcat_tab_viewing' );
		$form->addHeader( 'vcat_tab_viewing' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_videoTab', $this->_id ? $this->videoTab : 1, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_hideinfo', $this->bitoptions['hideinfo'], FALSE, array() ) );	
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_showAlphabet', $this->bitoptions['showAlphabet'] ? $this->bitoptions['showAlphabet'] : 0, FALSE ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_offDescription', $this->bitoptions['offDescription'] ? $this->bitoptions['offDescription'] : 0, FALSE, array( 'togglesOff' => array( 'vcat_bitoptions_noformat', 'vcat_contentformat' ) ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'vcat_bitoptions_noformat', $this->bitoptions['noformat'], FALSE, array(), NULL, NULL, NULL, 'vcat_bitoptions_noformat' ) );		
		$form->add( new \IPS\Helpers\Form\TextArea( 'vcat_contentformat', $this->_id ? $this->contentformat : '', FALSE, array( 'rows' => 10, 'placeholder' => "<strong>{content}</strong>" ), NULL, NULL, NULL, 'vcat_contentformat' ) );				
	}
	
	public function _saveClubForm( \IPS\Member\Club $club, $values )
	{
		foreach ( array( 'comments', 'rating', 'sharing', 'noformat', 'hideinfo', 'reviews', 'review_moderation', 'offDescription', 'showAlphabet' ) as $k )
		{
			if ( isset( $values[ "vcat_bitoptions_{$k}" ] ) )
			{
				$this->bitoptions[ $k ] = $values["vcat_bitoptions_{$k}"];
				unset( $values["vcat_bitoptions_{$k}"] );
			}
		}

		if ( $values['vcat_cfields'] )
		{
			$this->cfields = implode(",", $values['vcat_cfields']);
		}
		
		$this->videoTab = $values['vcat_videoTab'];
		$this->contentformat = $values['vcat_contentformat'];		
		$this->videoPerPage = (int) $values['vcat_videoPerPage'];		
		$this->view_in_popup = (int) $values['vcat_view_in_popup'];		
		$this->view_in_popup_device = $values['view_in_popup_device'] ? implode(',', $values['view_in_popup_device']) : null;		
		$this->tags_disabled = !$values['vcat_tags_disabled'];		
		$this->tags_noprefixes = !$values['vcat_tags_noprefixes'];		
		$this->tags_predefined = $values['vcat_tags_predefined'];		
		$this->tags_filter_on = (int) $values['vcat_tags_filter_on'];		
		$this->cfields_pos = $values['vcat_cfields_pos'];		
		
		if ( $values['club_node_name'] )
		{
			$this->title_furl = \IPS\Http\Url\Friendly::seoTitle( $values['club_node_name'] );
		}		
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		
		foreach ( array( 'vcat_title' => "videobox_category_{$this->id}" ) as $fieldKey => $langKey )
		{
			\IPS\Lang::deleteCustom( 'videobox', $langKey );
			\IPS\Lang::deleteCustom( 'core', $langKey ); // Remove custom lang for cloned catergory, because it was saved for core.
		}		
	}
	
	/**
	 * If there is any paid category
	 */
	public static function hasPaidCategory()
	{
		foreach( \IPS\videobox\Category::roots() as $cat )
		{
			if ( $cat->nexus_on == 1 )
			{
				return TRUE;
			}
			if ( $cat->hasChildren() )
			{
				return self::childCat( $cat );
			}
		}

		return FALSE;
	}	
	
	protected function childCat( $category )
	{	
		foreach ( $category->children() as $cat )
		{		
			if ( $cat->nexus_on == 1 )
			{
				return TRUE;
			}		
			if ( $cat->hasChildren() )
			{
				return self::childCat( $cat );
			}		
		}

		return FALSE;
	}	

	/**
	 * [Node] Does the currently logged in user have permission to mass move/delete content in this node?
	 *
	 * @return	bool
	 */
	public function canMassManageContent()
	{
		return TRUE;
	}
	
	/**
	 * Get all tags from videos in this category
	 */
	protected static $_tags = NULL;
	
	public function getAllTags()
	{
		if ( static::$_tags !== NULL )
		{
			return static::$_tags;
		}
		
		$tags = array();
		
		$where[] = array( "tag_meta_app=? AND tag_meta_area=?", 'videobox', 'video' );
		
		if ( $this->hasChildren() )
		{
			$catIDs = static::getChildCats( $this );
			$where[] = array( "tag_meta_parent_id IN (" . implode(",", $catIDs) . ")" );
		}
		else
		{
			$where[] = array( "tag_meta_parent_id=?", $this->id );
		}
		
		foreach( \IPS\Db::i()->select( 'DISTINCT(tag_text)', 'core_tags', $where, 'tag_text ASC' ) as $tag )
		{
			$tags[] = $tag;
		}

		static::$_tags = $tags;
		
		return static::$_tags;
	}
	
	/**
	 * Get id of sub categories
	 */	
	public static function getChildCats( $category )
	{	
		$listID = array();	
		$idchild = array();	
		$idchild[] = $category->id;
		
		foreach ( $category->children() as $cat )
		{		
			if ( $cat->hasChildren() )
			{
				$idchild[] = static::getChildCats( $cat );
				
			}
			$idchild[] = $cat->id;			
		}
		
		$iterator = new \RecursiveIteratorIterator( new \RecursiveArrayIterator( $idchild ) );
		$listID = iterator_to_array( $iterator, false );
		array_unique( $listID );
			
		return $listID;
	}	
	
	public function getAllCats( $includeClubs=false )
	{
		$data['normal'] = \IPS\videobox\Category::roots();
		
		if ( $includeClubs )
		{
			$permWhere[] = array( 'app=? and perm_type=?', 'videobox', 'category' );
					
			$permArr = \IPS\Member::loggedIn()->permissionArray();
			if ( ($key = array_search('cm', $permArr)) !== false ) {
				unset( $permArr[$key] );
			}
			
			$permWhere[] = array( '(' . \IPS\Db::i()->findInSet( 'perm_' . \IPS\videobox\Category::$permissionMap['read'], $permArr ) . ' OR ' . 'perm_' . \IPS\videobox\Category::$permissionMap['read'] . '=? )', '*' );

			foreach( \IPS\Db::i()->select( 'perm_type_id', 'core_permission_index', $permWhere ) as $c )
			{
				$ids[]	= $c;
			}
			
			if ( \count( $ids ) > 0 )
			{
				$where[] = array( 'vcat_id IN (' . implode(',', $ids) . ')' );
			}
			
			$where[] = array( 'vcat_club_id IS NOT NULL' );

			foreach( \IPS\Db::i()->select( '*', 'videobox_categories', $where, 'vcat_position ASC' ) as $row )
			{
				$cat = \IPS\videobox\Category::constructFromData( $row );
				if ( $cat->club() )
				{
					$data['clubs'][] = $cat;
				}
			}
		}
	
		return $data;
	}
	
	/**
	 * Can view lightbox
	 */
	public function canViewLightbox()
	{
		if ( $this->view_in_popup == 1 )
		{
			return TRUE;
		}
		
		return FALSE;
	}
	

	
	
	
	
	
}